import { AttributeNode } from '@vue/compiler-core';
import { DirectiveNode } from '@vue/compiler-core';
import { ElementNode } from '@vue/compiler-core';
import { InterpolationNode } from '@vue/compiler-core';
import { ParserOptions } from '@vue/compiler-core';
import { RootNode } from '@vue/compiler-core';
import { SimpleExpressionNode } from '@vue/compiler-core';
import { TemplateChildNode } from '@vue/compiler-core';
import { TextNode } from '@vue/compiler-core';
import { TransformOptions } from '@vue/compiler-core';

export declare interface BaseGenerateContext {
    code: string;
    push(code: string): void;
    nodeGenerate: NodeGenerate;
    propsType?: string;
}

export declare function compiler(template: string | RootNode, options?: CompilerOptions): {
    code: string;
};

export declare type CompilerOptions = ParserOptions & TransformOptions & GenerateContextOptions;

export declare type GenerateContext = BaseGenerateContext & GenerateUtil & GenFn;

export declare interface GenerateContextOptions {
    scopeId?: string | null;
    nodeGenerate?: NodeGenerate;
    genFn?: GenFn;
}

export declare interface GenerateUtil {
    isRoot(node: any): boolean;
    isNode(node: any): node is ElementNode;
    isText(node: any): node is TextNode;
    isSimpleExpr(node: any): node is SimpleExpressionNode;
    isExpr(node: any): node is InterpolationNode;
    isAttr(node: any): node is AttributeNode;
    isDirective(node: any): node is DirectiveNode;
}

export declare interface GenFn {
    genText(text: string, context: GenerateContext, node?: TextNode): void;
    genTextExpr(node: SimpleExpressionNode, context: GenerateContext): void;
    genPropsExpr(node: SimpleExpressionNode, context: GenerateContext): void;
    genStringValue(text: string, context: GenerateContext): void;
    genNode(node: ElementNode, context: GenerateContext): NodeGenerate;
    genNodeName(name: string, context: GenerateContext, node: TemplateChildNode): void;
    genProps(props: Array<AttributeNode | DirectiveNode>, context: GenerateContext, node: TemplateChildNode): void;
    genAttr(node: AttributeNode, context: GenerateContext): void;
    genDirective(node: DirectiveNode, context: GenerateContext): void;
    genPropsBind(node: DirectiveNode, context: GenerateContext): void;
    genPropsEvent(node: DirectiveNode, context: GenerateContext): void;
    genPropsCustomDirective(node: DirectiveNode, context: GenerateContext): void;
}

export declare type NodeGenerate = (node: RootNode | TemplateChildNode, context: GenerateContext) => void | NodeGenerate;

export { }
